@@ -8,7 +8,7 @@ struct vctrs_type_info vec_type_info(r_obj* x) {
   };
 
   switch (info.type) {
-  case vctrs_type_s3: info.proxy_method = vec_proxy_method(x); break;
+  case VCTRS_TYPE_s3: info.proxy_method = vec_proxy_method(x); break;
   default: info.proxy_method = r_null;
   }
   info.shelter = info.proxy_method;
@@ -66,21 +66,21 @@ static
 enum vctrs_type vec_base_typeof(r_obj* x, bool proxied) {
   switch (r_typeof(x)) {
   // Atomic types are always vectors
-  case R_TYPE_null: return vctrs_type_null;
-  case R_TYPE_logical: return vctrs_type_logical;
-  case R_TYPE_integer: return vctrs_type_integer;
-  case R_TYPE_double: return vctrs_type_double;
-  case R_TYPE_complex: return vctrs_type_complex;
-  case R_TYPE_character: return vctrs_type_character;
-  case R_TYPE_raw: return vctrs_type_raw;
+  case R_TYPE_null: return VCTRS_TYPE_null;
+  case R_TYPE_logical: return VCTRS_TYPE_logical;
+  case R_TYPE_integer: return VCTRS_TYPE_integer;
+  case R_TYPE_double: return VCTRS_TYPE_double;
+  case R_TYPE_complex: return VCTRS_TYPE_complex;
+  case R_TYPE_character: return VCTRS_TYPE_character;
+  case R_TYPE_raw: return VCTRS_TYPE_raw;
   case R_TYPE_list:
     // Bare lists and data frames are vectors
-    if (!r_is_object(x)) return vctrs_type_list;
-    if (is_data_frame(x)) return vctrs_type_dataframe;
+    if (!r_is_object(x)) return VCTRS_TYPE_list;
+    if (is_data_frame(x)) return VCTRS_TYPE_dataframe;
     // S3 lists are only vectors if they are proxied
-    if (proxied || r_inherits(x, "list")) return vctrs_type_list;
+    if (proxied || r_inherits(x, "list")) return VCTRS_TYPE_list;
     // fallthrough
-  default: return vctrs_type_scalar;
+  default: return VCTRS_TYPE_scalar;
   }
 }
 
@@ -123,7 +123,7 @@ bool vec_is_vector(r_obj* x) {
   }
 
   struct vctrs_proxy_info info = vec_proxy_info(x);
-  return info.type != vctrs_type_scalar;
+  return info.type != VCTRS_TYPE_scalar;
 }
 
 // [[ register() ]]
@@ -153,9 +153,9 @@ r_obj* vctrs_typeof(r_obj* x, r_obj* dispatch) {
 
 enum vctrs_type vec_typeof(r_obj* x) {
   // Check for unspecified vectors before `vec_base_typeof()` which
-  // allows vectors of `NA` to pass through as `vctrs_type_logical`
+  // allows vectors of `NA` to pass through as `VCTRS_TYPE_logical`
   if (vec_is_unspecified(x)) {
-    return vctrs_type_unspecified;
+    return VCTRS_TYPE_unspecified;
   }
 
   if (!r_is_object(x) || r_class(x) == r_null) {
@@ -166,10 +166,10 @@ enum vctrs_type vec_typeof(r_obj* x) {
   // data frames are treated as S3 to give them a chance to be proxied
   // or implement their own methods for cast, type2, etc.
   if (is_bare_data_frame(x)) {
-    return vctrs_type_dataframe;
+    return VCTRS_TYPE_dataframe;
   }
 
-  return vctrs_type_s3;
+  return VCTRS_TYPE_s3;
 }
 
 r_no_return
@@ -179,18 +179,18 @@ void stop_unimplemented_vctrs_type(const char* fn, enum vctrs_type type) {
 
 const char* vec_type_as_str(enum vctrs_type type) {
   switch (type) {
-  case vctrs_type_null:         return "null";
-  case vctrs_type_unspecified:  return "unspecified";
-  case vctrs_type_logical:      return "logical";
-  case vctrs_type_integer:      return "integer";
-  case vctrs_type_double:       return "double";
-  case vctrs_type_complex:      return "complex";
-  case vctrs_type_character:    return "character";
-  case vctrs_type_raw:          return "raw";
-  case vctrs_type_list:         return "list";
-  case vctrs_type_dataframe:    return "dataframe";
-  case vctrs_type_s3:           return "s3";
-  case vctrs_type_scalar:       return "scalar";
+  case VCTRS_TYPE_null:         return "null";
+  case VCTRS_TYPE_unspecified:  return "unspecified";
+  case VCTRS_TYPE_logical:      return "logical";
+  case VCTRS_TYPE_integer:      return "integer";
+  case VCTRS_TYPE_double:       return "double";
+  case VCTRS_TYPE_complex:      return "complex";
+  case VCTRS_TYPE_character:    return "character";
+  case VCTRS_TYPE_raw:          return "raw";
+  case VCTRS_TYPE_list:         return "list";
+  case VCTRS_TYPE_dataframe:    return "dataframe";
+  case VCTRS_TYPE_s3:           return "s3";
+  case VCTRS_TYPE_scalar:       return "scalar";
   }
   never_reached("vec_type_as_str");
 }
