@@ -25,6 +25,7 @@ bool vec_is_unspecified(SEXP x);
 #include "dictionary.h"
 #include "dim.h"
 #include "equal.h"
+#include "expand.h"
 #include "hash.h"
 #include "lazy.h"
 #include "match-compare.h"
@@ -44,7 +45,9 @@ bool vec_is_unspecified(SEXP x);
 #include "ptype.h"
 #include "ptype2-dispatch.h"
 #include "ptype2.h"
+#include "rep.h"
 #include "runs.h"
+#include "set.h"
 #include "shape.h"
 #include "size-common.h"
 #include "size.h"
